# cmdb.tcl---directory browser

# Fri Jul 15 09:46:44 BST 1994, SCL
#   added check for OS version due to different output from ls


set started 0

set frame CMDB
set frametitle "ColourMap Directory Browser"
set icontitle CMBrowser

set defdir [file join $env(HOME) .midas colourmaps]
set directory $defdir
set file ""
set savefile ""

loadsource sb.common 

proc make-frame {} {
   global env

   :frame [self frame-name] :label "[self frame-title]" :show-footer T  \
      :icon-image "@$env(ICONHOME)/spectrum"  \
      :icon-name [self icon-title]

    :panel P1 :layout V :pady 5 :padx 10
     :layout H
      :text DIRECTORY :label "Directory" :w 40 :shifters T
      :hskip 100
      :button UP; :button DOWN

    :panel P2 :layout V :padx 10
      :list DIRECTORYLIST :rows 10 :cols 80 :exclusive T :double-click T

    :panel P3 :layout V :pady 5 :padx 10
     :layout H; :row-gap 10; :col-gap 5
      :button LOAD;  :button SAVE; :col-gap 0
      :text FILE :label "" :w 15 :deaf T
      :hskip 10; :col-gap 0
      :button SAVE-AS :label "Save As"
      :text SAVEFILE :label "" :w 15
      :hskip 50
      :button REDISPLAY; :button HELP 

    :show T
}

proc double-click arg {
   global debug
   if {$debug} then {puts stderr "double-click with argument \{$arg\}"}
   set w [lindex $arg 2]

   switch -glob $w {
        DIRECTORYLIST        {click-down-or-load}
	default     {puts stdout $arg}
   }
}

proc click-savefile f {
   global savefile
   set savefile $f
}

proc click-load {} {
   global selection directorylist directory file
   clear-footer
   if {[llength $selection] == 0} then {
      set-footer "no colourmap file selected!!!"
      return
   }
   if {[llength $selection] > 1} then {
      set-footer "multiple selections!!!"
      return
   }
   set target [lindex $directorylist $selection]
   set link [last $target]
   set type [file type [file join $directory $link]]
   if {$type == "link"} then {
      set link [file readlink $link]
      set type [file type [file join $directory $link]]
   }
   if {$type == "directory"} then {
      set-footer "can't load a directory!!!"
      return
   }
   set file [last $target]
   :modify FILE :value "$file"
   load-colourmap-file $directory $file
}

proc click-down-or-load {} {
   global selection directorylist directory file
   clear-footer
   if {[llength $selection] == 0} then {
      set-footer "no colourmap file selected!!!"
      return
   }
   if {[llength $selection] > 1} then {
      set-footer "multiple selections!!!"
      return
   }
   set target [lindex $directorylist $selection]
   set file [last $target]
   if {[file type [file join $directory $file]] == "link"} then {
      set file [file readlink [file join $directory $file]]
   }
   if {[file type [file join $directory $file]] != "directory"} then {
       :modify FILE :value "$file"
       load-colourmap-file $directory $file
       return
   }
   if {"$file" == "."} then {click-redisplay; return}
   if {"$file" == ".."} then {click-up; return}
   if {$directory == "/"} then {set directory ""}
   set directory [file join $directory $file]
   click-redisplay
   set selection {}
}

proc click-save {} {
   global directory file
   clear-footer
   if {"$file" == ""} then {
      midas-warning "You haven't loaded a colourmap file yet!!!"
      set-footer "you haven't loaded a colourmap file yet!!!"
      return
   }
   save-colourmap-file $directory $file
}

proc click-save-as {} {
   global directory savefile
   clear-footer
   if {"$savefile" == ""} then {
      midas-warning "You haven't given a new file name!!!"
      set-footer "you haven't given a new file name!!!"
      return
   }
   save-colourmap-file $directory $savefile
   click-redisplay
}

proc start args {
   global defdir started
    if {$started == 0} then {
	make-frame
	set z [catch {:file mkdir $defdir}]
	set started 1
    } else {open-frame}
   click-redisplay
}

proc load-colourmap-file {dir file} {
   set z [catch {set cmap [read-colourmap-file [file join $dir $file]]} m]
   if $z!=0 then {set-footer $m; return}
   set l [expr [llength $cmap]-1]
   set cmap [lreplace $cmap $l $l [lindex $cmap $l]]
   inform opt CM.set-colours [list $cmap]
}

proc save-colourmap-file {dir file} {
   set z [catch {set cmap [inform opt inquire preferences CM.colours]} m]
   if $z!=0 then {set-footer $m; return}
   set z [catch {write-colourmap-file [file join $dir $file] $cmap} m]
   if $z!=0 then {set-footer $m; return}
}

proc read-colourmap-file fn {
   set z [catch {open $fn r} f]
   if {$z != 0} then {
       midas-warning "Error reading colourmap file\n$f"
       error $f
       return
   }
   set cmap [read $f]
   close $f
   return $cmap
}

proc write-colourmap-file {fn cmap} {
   set z [catch {open $fn w} f]
   if {$z != 0} then {
       midas-warning "Error saving colourmap file\n$f"
       error $f
       return
   }
   puts $f $cmap
   close $f
}
