#----------------------------------------------------------------------SP
set SP(frame-name) SP
set SP(frame-title) "Spectrum Plotting Options"
set SP(icon-title) optSP

set SP(normnames) {linear log}
set SP(values) {"1" "2" "3" "4" "5" "6" "7" "8" "9"}
set SP(strips) 0
set SP(scale) 0
set SP(norm) 0
set SP(peaks) 0
set SP(low-chan) 0
set SP(high-chan) 0
set SP(gains) {}
set SP(selection) 0
set SP(paper) 0
set SP(papernames) {}
set SP(printer) ""
set SP(printer) ""


proc SP.make-frame {} {
   global env SP

   :frame $SP(frame-name) :label "$SP(frame-title)" :show-footer T :toplevel T \
        :icon-image "@$env(ICONHOME)/spectrum"  \
           :icon-name $SP(icon-title) :resize F

   :panel SP-OPTIONS :layout T :padx 5 :pady 5
    :layout H; :row-gap 5

   :slider SP-LOW   :label "Low Channel:"  :w 8 :valuex 18c :min 0 :max 8192 :command "SP.opt low-chan"
   :next-row T
   :slider SP-HIGH  :label "High Channel:" :w 8 :valuex 18c :min 0 :max 8192 :command "SP.opt high-chan"
   :next-row T
   :slider SP-FWHM  :label "Scale (chan/inch):" :w 8 :valuex 18c :min 0 :max 1000 :command "SP.opt scale"
   :next-row T
   :message    SP-L1  :label "Strips:"  :valuex 16c
   :menubutton SP-STRIPS  :label "$SP(strips)" :w 15 :strings $SP(values) :command "SP.opt2 strips"
   :next-row T
   :message    SP-L2  :label "Normalisation:"  :valuex 16c
   :menubutton SP-NORM    :label "$SP(norm)" :w 15 :strings $SP(normnames) :command "SP.opt2 norm"
   :next-row T
   :checkbox SP-PEAKS :label "Plot peaks" :valuex 16c :value $SP(peaks) :command "SP.opt peaks"
   :next-row T
   :message    SP-L3  :label "Select Printer" :valuex 16c
   :button SP-PRINTER  :label "" :w 18  :command "inform opt-misc start"
   :next-row T
   :list SP-LIST  :cols 40 :rows 4 :command SP.region

   :panel SP-BUTTONS :layout B :padx 5
    :layout H; :col-gap 10
   :button SP-RESET :label Reset :command SP.reset
   :button SP-TAKE  :label Take  :command SP.take
   :button SP-SPLIT :label Split :command SP.split
   :button SP-JOIN  :label Join  :command SP.join
   :button SP-PLOT  :label Plot  :command SP.plot
   :next-row T
   :button SP-SAVE  :label "Save as defaults" :command SP.save
   :button SP-REDISPLAY :label Redisplay :command SP.repaint
   :hskip 30
   :button SP-HELP  :label Help :command click-help

   :show T

   SP.repaint
}

proc SP.opt {opt args} {
   global debug SP
   if {$debug} then {insert-debug "SP.opt called with opt= $opt  args= $args"}
   set SP($opt) [second $args]
   SP.announce
}

proc SP.opt2 {opt args} {
   global debug SP
   if {$debug} then {insert-debug "SP.opt2 called with opt= $opt  args= $args"}
   set SP($opt) [second $args]
   :modify [first $args] :label [second $args]
   SP.announce
}

proc SP.repaint {} {
   global debug SP
   if {$debug} then {insert-debug "SP.repaint called"}
   SP.unpack
   :modify SP-LIST :empty T :strings [SP.format-regions [get SP gains]]
   :modify SP-LOW  :value $SP(low-chan)
   :modify SP-HIGH :value $SP(high-chan)
   :modify SP-FWHM :value $SP(scale)
   :modify SP-STRIPS :label $SP(strips)
   :modify SP-NORM   :label $SP(norm)
   :modify SP-PEAKS :value $SP(peaks)
   :modify SP-PRINTER :label $SP(printer)
   set-footer SP "plot will fill [SP.calc-strips] strips with these options"
}

proc SP.format-regions l {
   set ll {}
   foreach f $l {
      set x0 [first $f]
      set x1 [second $f]
      set y0 [third $f]
      set y1 [fourth $f]
      lappend ll [format "%8d %8d     %8d %8d" $x0 $x1 $y0 $y1]
   }
   return $ll
}

proc SP.region {args} {
   global debug SP frame
   if {$debug} then {insert-debug "SP.region called with args= $args"}

   set selection [third $args]
   if {$selection==""} then {put SP selection -1; return }
   set regionlist [get SP gains]
   set regionstrings [SP.format-regions $regionlist]
   put SP selection $selection
   if {$selection<0} then {return}
   set r [lindex $regionstrings $selection]
   inform sd zoom-to-region $r
}

proc SP.split {} {
   global debug SP
   if {$debug} then {insert-debug "SP.split called"}
   set selection [get SP selection]
   if {$selection<0} then {set-footer SP "you haven't selected a region to split!!!"; return}
   SP.split-at-tag [inform sd get-tag-and-pop]
}

proc SP.split-at-tag {args} {
   global debug SP
   if {$debug} then {insert-debug "SP.split-at-tag called with args= $args"}
   set regionlist [get SP gains]
   set selection [get SP selection]
   if {$selection<0} then {set-footer SP "you haven't selected a region!!!"; return}
   set c $args
# "{}" coz of bug in transparent???
   if {$args=="{}"} then {set-footer SP "you haven't tagged a split channel!!!"; return}
   set region [lindex $regionlist $selection]
   if {$c<=[first $region] || $c>=[second $region]} then \
      {set-footer SP "split channel $c not in this region!!!"; return}
   set region1 [list $c [second $region] [third $region] [fourth $region]]
   set region [list [first $region] $c [third $region] [fourth $region]]
   put SP gains [lreplace $regionlist $selection $selection $region $region1]
   apply SP
   redisplay SP
   inform sd zoom-to-region $region
}

proc SP.join {} {
   global debug SP
   if {$debug} then {insert-debug "SP.join called"}
   set selection [get SP selection]
   if {$selection<0} then {set-footer SP "you haven't selected a first region!!!"; return}
   set selection1 [expr $selection+1]
   set regionlist [get SP gains]
   if {[llength $regionlist]<=$selection1} then {return}
   set region [lindex $regionlist $selection]
   set region1 [lindex $regionlist $selection1]
   set region [list [first $region] [second $region1] [third $region] [fourth $region]]
   put SP gains [lreplace $regionlist $selection $selection1 $region]
   apply SP
   redisplay SP
   inform sd zoom-to-region $region
}

proc SP.take {} {
   global debug SP
   if {$debug} then {insert-debug "SP.take called"}
   set selection [get SP selection]
   if {$selection<0} then {set-footer SP "you haven't selected a region!!!"; return}
   SP.take-gain [inform sd get-zoom-region]
}

proc SP.take-gain r {
   global debug SP
   if {$debug} then {insert-debug "SP.take-gain called with r= $r"}
   set regionlist [get SP gains]
   set selection [get SP selection]
   if {$selection<0} then {set-footer SP "you haven't selected a region!!!"; return}
#   set region [lindex $regionlist $selection]
   set region [list [first $r] [second $r] [third $r] [fourth $r]]
   put SP gains [lreplace $regionlist $selection $selection $region]
   apply SP
   redisplay SP
}

proc SP.reset {} {
   global debug SP
   if {$debug} then {insert-debug "SP.reset called"}
   SP.reset-gain [inform sd get-zoom-region]
}

proc SP.reset-gain r {
   global debug SP
   if {$debug} then {insert-debug "SP.reset-gain called with r= $r"}
   set region $r
   set regionlist [list $region]
   put SP gains $regionlist
   put SP selection 0
   put SP low-chan [first $region]
   put SP high-chan [second $region]
   apply SP
   redisplay SP
   inform sd zoom-to-region $region
}

proc SP.calc-strips {} {
   global debug SP
   if {$debug} then {insert-debug "SP.calc-strips called"}
    set a [format "%1.1f" [expr ([get SP high-chan]-[get SP low-chan]+1)/10.00/[get SP scale] +0.5]]
    string range $a 0 0
}

proc SP.plot {} {
   global debug SP
   if {$debug} then {insert-debug "SP.plot called"}
   inform sd "click-plot [transparent {}]"
}

proc SP.unpack {} {
   global debug SP
   if {$debug} then {insert-debug "SP.unpack called"}
   foreach p [get-eg-option specplot] {
      switch [first $p] {
      norm {set SP(norm) [second $p]}
      low-chan {set SP(low-chan) [second $p]}
      high-chan {set SP(high-chan) [second $p]}
      strips {set SP(strips) [second $p]}
      scale {set SP(scale) [second $p]}
      peaks {set SP(peaks) [second $p]}
      gains {
          set g {}
          foreach t [second $p] {
              lappend g [list [first $t] [third $t] [second $t] [fourth $t]]
          }
          set SP(gains) $g
      }
      paper {set SP(paper) [second $p]}
      }
   }
   set SP(printer) [get-eg-option printer]
}

proc SP.announce {} {
   global debug SP
   if {$debug} then {insert-debug "SP.announce called"}
    set g {}
    foreach t [get SP gains] {
        lappend g [list [first $t] [third $t] [second $t] [fourth $t]]
    }
   set-eg-option specplot [list \
        [list low-chan [get SP low-chan]] \
        [list high-chan [get SP high-chan]] \
        [list strips [get SP strips]] \
        [list norm [get SP norm]] \
        [list scale [get SP scale]] \
        [list peaks [get SP peaks]] \
        [list gains $g] \
        [list paper [get SP paper]]]
   redisplay SP
}

proc SP.save {} {edit-defaults specplot "set-eg-option specplot \{[get-eg-option specplot]\}"}
